import { useDomain } from "@/hook/useDomain";
import { BrowserRouter, Route, Routes, useLocation } from "react-router-dom";
import { baseRoutes } from "./baseRoutes";
import { roleRoutes } from "./roleRoutes";
import { RouteGuard } from "./Guard";
import { observer } from "mobx-react-lite";
import { ROLE_NAME, defaultDocumentTitle } from "@/constant";
import { pathToRegexp } from "path-to-regexp";

export type ExpandRouteProps = {
  title?: string;
};

export const RouterElements = observer(() => {
  const { userDomainService } = useDomain();
  const { user } = userDomainService;
  const location = useLocation();

  const roleNames = user.roles?.map(
    (r: { name: string }) => r.name
  ) as ROLE_NAME[];

  /**
   * 根据角色生成路由
   */
  const currentRoutes = roleNames?.map((name) => roleRoutes[name]).flat() ?? [];

  /**
   * document.title
   */
  const currentRoute = [...currentRoutes, ...baseRoutes].find((r) =>
    pathToRegexp(r.path!).test(location.pathname)
  );
  if (currentRoute) document.title = currentRoute.title || defaultDocumentTitle;

  return (
    <RouteGuard ignorePaths={baseRoutes.map((r) => r.path!)}>
      <Routes>
        {currentRoutes.map((r) => (
          <Route key={r.path} path={r.path} element={r.element} />
        ))}
        {baseRoutes.map((r) => (
          <Route key={r.path} path={r.path} element={r.element} />
        ))}
      </Routes>
    </RouteGuard>
  );
});
